@@ -2,7 +2,7 @@
 
 /**
  * dmarc-srg - A php parser, viewer and summary report generator for incoming DMARC reports.
- * Copyright (C) 2022 Aleksey Andreev (liuch)
+ * Copyright (C) 2022-2025 Aleksey Andreev (liuch)
  *
  * Available at:
  * https://github.com/liuch/dmarc-srg
@@ -36,7 +36,7 @@ namespace Liuch\DmarcSrg\Exception;
  */
 class RuntimeException extends \RuntimeException
 {
-    public function __construct(string $message = '', int $code = -1, \Throwable $previous = null)
+    public function __construct(string $message = '', int $code = -1, ?\Throwable $previous = null)
     {
         parent::__construct($message, $code, $previous);
     }
