@@ -1,6 +1,6 @@
 /**
  * dmarc-srg - A php parser, viewer and summary report generator for incoming DMARC reports.
- * Copyright (C) 2020 Aleksey Andreev (liuch)
+ * Copyright (C) 2020-2025 Aleksey Andreev (liuch)
  *
  * Available at:
  * https://github.com/liuch/dmarc-srg
@@ -688,7 +688,8 @@ class ITableColumn extends ITableCell {
 }
 
 class Toolbar {
-	constructor() {
+	constructor(label) {
+		this._label   = label;
 		this._element = null;
 		this._items   = [];
 		this._spacer  = {}; // Just a unique value
@@ -697,24 +698,44 @@ class Toolbar {
 	element() {
 		if (!this._element) {
 			const el = document.createElement("div");
-			el.classList.add("toolbar");
-			this._element = el;
-			let spacer  = false
+			el.role = "toolbar";
+			el.ariaLabel = this._label;
+			let spacer = false
+			let first  = true;
 			for (const it of this._items) {
 				if (it === this._spacer) {
-					spacer = this._spacer;
+					spacer = true;
 					continue;
 				}
 				if (spacer) {
 					it.classList.add("spacer-left");
 					spacer = false;
 				}
-				if (it instanceof ToolbarButton) {
-					el.append(it.element());
-				} else {
-					el.append(it);
+				let te = (it instanceof ToolbarButton) ? it.element() : it;
+				el.append(te);
+				if (first && te.tabIndex === -1) {
+					first = false;
+					te.tabIndex = 0;
 				}
 			}
+			el.addEventListener("keydown", event => {
+				const target = event.target;
+				switch (event.code) {
+					case "ArrowLeft":
+						this._focusPreviousItem(event.target);
+						break;
+					case "ArrowRight":
+						this._focusNextItem(event.target);
+						break;
+					case "Home":
+						this._focusFirstItem();
+						break;
+					case "End":
+						this._focusLastItem();
+						break;
+				}
+			});
+			this._element = el;
 		}
 		return this._element;
 	}
@@ -728,6 +749,59 @@ class Toolbar {
 		this._items.push(this._spacer);
 		return this;
 	}
+
+	_focusFirstItem() {
+		const first = this._element.firstElementChild;
+		if (!first.hasAttribute("tabindex")) {
+			this._focusNextItem(first);
+			return;
+		}
+		this._resetTabindexValues();
+		first.tabIndex = 0;
+		first.focus();
+	}
+
+	_focusLastItem() {
+		const last = this._element.lastElementChild;
+		if (!last.hasAttribute("tabindex")) {
+			this._focusPreviousItem(last);
+			return;
+		}
+		this._resetTabindexValues();
+		last.tabIndex = 0;
+		last.focus();
+	}
+
+	_focusNextItem(cItem) {
+		let next = cItem;
+		while ((next = next.nextElementSibling)) {
+			if (next.hasAttribute("tabindex")) {
+				this._resetTabindexValues();
+				next.tabIndex = 0;
+				next.focus();
+				break;
+			}
+		}
+	}
+
+	_focusPreviousItem(cItem) {
+		let prev = cItem;
+		while ((prev = prev.previousElementSibling)) {
+			if (prev.hasAttribute("tabindex")) {
+				this._resetTabindexValues();
+				prev.tabIndex = 0;
+				prev.focus();
+				break;
+			}
+		}
+
+	}
+
+	_resetTabindexValues() {
+		this._element.querySelectorAll('[tabindex="0"]').forEach(el => {
+			el.tabIndex = -1;
+		});
+	}
 }
 
 class ToolbarButton {
@@ -741,16 +815,45 @@ class ToolbarButton {
 	element() {
 		if (!this._element) {
 			const el = document.createElement("button");
+			el.type = "button";
 			el.tabIndex = -1;
-			if (this._content) el.append(this._content);
-			if (this._title) el.title = this._title;
+			let ce = null
+			if (this._content) {
+				ce = el.appendChild((typeof(this._content) === "string") && this._getSVG() || this._content);
+			}
+			if (this._title) {
+				const popup = document.createElement("span");
+				popup.classList.add("popup-label");
+				el.appendChild(popup).textContent = this._title;
+				if (ce && ce.nodeName.toUpperCase() === "SVG") ce.setAttribute("aria-hidden", true);
+			}
 			if (this._onclick) el.addEventListener("click", this._onclick);
 			this._element = el;
 		}
 		return this._element;
 	}
 
-	static svgIcon(view_box, html) {
+	_getSVG() {
+		switch (this._content) {
+			case "info_icon":
+				return this._svgIcon(
+					'0 0 16 16',
+					'<path d="m8.93 6.588-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM9 4.5a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'
+				);
+			case "filter_icon":
+				return this._svgIcon(
+					'0 1 15 15',
+					'<path d="M1.5 1.5A.5.5 0 0 1 2 1h12a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.128.334L10 8.692V13.5a.5.5 0 0 1-.342.474l-3 1A.5.5 0 0 1 6 14.5V8.692L1.628 3.834A.5.5 0 0 1 1.5 3.5zm1 .5v1.308l4.372 4.858A.5.5 0 0 1 7 8.5v5.306l2-.666V8.5a.5.5 0 0 1 .128-.334L13.5 3.308V2z"/>'
+				);
+			case "columns_icon":
+				return this._svgIcon(
+					'0 0 17 17',
+					'<path d="M0 1.5A1.5 1.5 0 0 1 1.5 0h13A1.5 1.5 0 0 1 16 1.5v13a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 14.5zM1.5 1a.5.5 0 0 0-.5.5v13a.5.5 0 0 0 .5.5H5V1zM10 15V1H6v14zm1 0h3.5a.5.5 0 0 0 .5-.5v-13a.5.5 0 0 0-.5-.5H11z"/>'
+				);
+		}
+	}
+
+	_svgIcon(view_box, html) {
 		const svg = document.createElementNS("http://www.w3.org/2000/svg", "svg");
 		svg.setAttribute("focusable", "false");
 		svg.setAttribute("fill", "currentColor");
@@ -758,14 +861,6 @@ class ToolbarButton {
 		svg.innerHTML = html;
 		return svg;
 	}
-
-	static filterIcon() {
-		return this.svgIcon('0 1 15 15', '<path d="M1.5 1.5A.5.5 0 0 1 2 1h12a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.128.334L10 8.692V13.5a.5.5 0 0 1-.342.474l-3 1A.5.5 0 0 1 6 14.5V8.692L1.628 3.834A.5.5 0 0 1 1.5 3.5zm1 .5v1.308l4.372 4.858A.5.5 0 0 1 7 8.5v5.306l2-.666V8.5a.5.5 0 0 1 .128-.334L13.5 3.308V2z"/>');
-	}
-
-	static columnsIcon() {
-		return this.svgIcon('0 0 17 17', '<path d="M0 1.5A1.5 1.5 0 0 1 1.5 0h13A1.5 1.5 0 0 1 16 1.5v13a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 14.5zM1.5 1a.5.5 0 0 0-.5.5v13a.5.5 0 0 0 .5.5H5V1zM10 15V1H6v14zm1 0h3.5a.5.5 0 0 0 .5-.5v-13a.5.5 0 0 0-.5-.5H11z"/>');
-	}
 }
 
 class ModalDialog {
@@ -773,84 +868,89 @@ class ModalDialog {
 		this._params   = params;
 		this._element  = null;
 		this._title    = null;
+		this._messages = null;
+		this._alert    = null;
+		this._wait     = null;
 		this._buttons  = [];
 		this._content  = null;
-		this._first    = null;
-		this._last     = null;
 		this._result   = null;
 		this._callback = null;
 	}
 
 	element() {
 		if (!this._element) {
-			let ovl = document.createElement("div");
-			ovl.setAttribute("class", "dialog-overlay hidden");
-			let dlg = document.createElement("div");
-			dlg.setAttribute("class", "dialog");
-			let con = document.createElement("div");
-			con.setAttribute("class", "container");
-			this._title = document.createElement("div");
-			this._title.setAttribute("class", "title");
-			{
-				let tt = document.createElement("div");
-				tt.setAttribute("class", "title-text");
-				tt.appendChild(document.createTextNode(this._params.title || ""));
-				this._title.appendChild(tt);
-			}
-			let that = this;
-			{
-				let cbt = document.createElement("button");
-				cbt.setAttribute("type", "button");
-				cbt.setAttribute("class", "close-btn");
-				cbt.appendChild(document.createTextNode("x"));
-				this._title.appendChild(cbt);
-				this._buttons = [ cbt ];
-				cbt.addEventListener("click", function(event) {
-					that.hide();
-				});
-			}
-			con.appendChild(this._title);
-			let frm = document.createElement("form");
-			this._content = document.createElement("div");
-			frm.appendChild(this._content);
-			let bdv = document.createElement("div");
-			bdv.setAttribute("class", "dialog-buttons");
-			this._add_buttons(bdv);
-			frm.appendChild(bdv);
-			con.appendChild(frm);
-			dlg.appendChild(con);
-			ovl.appendChild(dlg);
+			const ovl = document.createElement("div");
+			ovl.classList.add("dialog-overlay", "hidden");
 			this._element = ovl;
+			const dlg = ovl.appendChild(document.createElement("div"));
+			dlg.role = "dialog";
+			dlg.ariaModal = true;
+			dlg.tabIndex = -1; // To catch keydown events
+			dlg.classList.add("dialog");
+			const con = dlg.appendChild(document.createElement("div"));
+			con.classList.add("container");
+			this._title = con.appendChild(document.createElement("div"));
+			this._title.classList.add("title");
+			{
+				const tt = this._title.appendChild(document.createElement("div"));
+				tt.classList.add("title-text");
+				tt.textContent = this._params.title || "";
+				if (this._params.title) dlg.setAriaLabelledBy(tt);
+			}
+			{
+				const cbt = this._title.appendChild(document.createElement("button"));
+				cbt.type = "button";
+				cbt.ariaLabel = "Close";
+				cbt.classList.add("close-btn");
+				cbt.textContent = "x";
+				this._buttons = [ cbt ];
+				cbt.addEventListener("click", event => this.hide());
+			}
+			const frm = con.appendChild(document.createElement("form"));
+			frm.classList.add("vertical-content");
+			this._content = frm.appendChild(document.createElement("div"));
+			const bdv = frm.appendChild(document.createElement("div"));
+			bdv.classList.add("dialog-buttons");
+			this._add_buttons(bdv);
 			this._gen_content();
-			this._update_first_last();
-			this._element.addEventListener("click", function(event) {
-				if (event.target === this && that._params.overlay_click !== "ignore") {
-					that.hide();
+			this._element.addEventListener("click", event => {
+				if (event.target === event.currentTarget && this._params.overlay_click !== "ignore") {
+					this.hide();
 				}
 			});
-			frm.addEventListener("keydown", function(event) {
-				if (event.key == "Tab") {
-					if (!event.shiftKey) {
-						if (event.target == that._last) {
-							that._first.focus();
+			this._element.addEventListener("keydown", event => {
+				switch (event.code) {
+					case "Tab":
+						{
+							const els = this._get_focusable_elements();
+							const lfe = [ els[0], els[els.length - 1] ];
+							switch (lfe.indexOf(event.target)) {
+								case -1:
+									return;
+								case 0:
+									if (!event.shiftKey) return;
+									lfe[1].focus();
+									break;
+								case 1:
+									if (event.shiftKey) return;
+									lfe[0].focus();
+									break;
+							}
 							event.preventDefault();
 						}
-					}
-					else {
-						if (event.target == that._first) {
-							that._last.focus();
-							event.preventDefault();
-						}
-					}
+						break;
+					case "Esc":
+					case "Escape":
+						event.preventDefault();
+						this.hide();
+						break;
 				}
 			});
-			frm.addEventListener("submit", function(event) {
+			frm.addEventListener("submit", event => {
 				event.preventDefault();
-				that._submit();
-			});
-			frm.addEventListener("reset", function(event) {
-				that._reset();
+				this._submit();
 			});
+			frm.addEventListener("reset", event => this._reset());
 		}
 		return this._element;
 	}
@@ -858,26 +958,74 @@ class ModalDialog {
 	show() {
 		this.element();
 		this._result = null;
-		this._title.querySelector("button.close-btn").classList.add("active");
 		this._element.classList.remove("hidden");
-		if (this._first) {
-			this._first.focus();
-		}
+		this.focus();
 
-		let that = this;
-		return new Promise(function(resolve, reject) {
-			that._callback = resolve;
+		return new Promise((resolve, reject) => {
+			this._callback = resolve;
 		});
 	}
 
 	hide() {
-		if (this._element) {
-			this._title.querySelector("button.close-btn").classList.remove("active");
-			this._element.classList.add("hidden");
-		}
+		if (this._element) this._element.classList.add("hidden");
 		this._callback && this._callback(this._result);
 	}
 
+	focus() {
+		this.element();
+		const els = this._get_focusable_elements(2);
+		switch (els.length) {
+			case 2:
+				if (els[0].classList.contains("close-btn")) {
+					els[1].focus();
+					break;
+				}
+			case 1:
+				els[0].focus();
+				break;
+			default:
+				this._element.querySelector('[role="dialog"]').focus();
+				break;
+		}
+	}
+
+	display_status(type, text) {
+		if (type && !text) {
+			type == "error" && this._alert && this._alert.replaceChildren();
+			type == "wait" && this._wait && this._wait.replaceChildren();
+		} else {
+			this._alert && this._alert.replaceChildren();
+			this._wait && this._wait.replaceChildren();
+		}
+		if (!text) return;
+
+		const t_el = document.createElement("p");
+		t_el.textContent = text;
+		if (type == "error") {
+			if (!this._alert) {
+				this._alert = this._make_msg_container();
+				this._alert.role = "alert";
+			}
+			t_el.classList.add("error-message");
+			this._alert.append(t_el);
+		} else if (type == "wait") {
+			if (!this._wait) {
+				this._wait = this._make_msg_container();
+			}
+			t_el.classList.add("wait-message");
+			this._wait.append(t_el);
+		}
+	}
+
+	_make_msg_container() {
+		if (!this._messages) {
+			this._messages = document.createElement("div");
+			const btns = this.element().querySelector("form .dialog-buttons");
+			btns.parentElement.insertBefore(this._messages, btns);
+		}
+		return this._messages.appendChild(document.createElement("div"));
+	}
+
 	_add_buttons(container) {
 		let bl = this._params.buttons || [];
 		bl.forEach(function(bt) {
@@ -932,18 +1080,19 @@ class ModalDialog {
 	_gen_content() {
 	}
 
-	_update_first_last() {
-		this._first = null;
-		this._last  = null;
-		for (const el of this._element.querySelector("form").querySelectorAll("input, select, button, a[href], [tabindex]")) {
+	_get_focusable_elements(max) {
+		const res = [];
+		if (!max) max = -1;
+		for (const el of this._element.querySelectorAll("input, select, button, a[href], [tabindex]")) {
 			const ti = el.tabIndex;
 			if (!isNaN(ti) && ti >= 0 && !el.disabled) {
 				if (window.getComputedStyle(el, null).display !== "none") {
-					if (!this._first) this._first = el;
-					this._last = el;
+					res.push(el);
+					if (!(--max)) break;
 				}
 			}
 		}
+		return res;
 	}
 
 	_submit() {
@@ -1120,8 +1269,7 @@ class ReportFilterDialog extends ModalDialog {
 		for (let i = 0; i < list.length; ++i) {
 			list[i].disabled = !enable;
 		}
-		this._update_first_last();
-		if (this._first) this._first.focus();
+		this.focus();
 	}
 
 	_update_ui() {
@@ -1182,25 +1330,29 @@ class Multiselect extends HTMLElement {
 		this._items    = [];
 		this._aitems   = [];
 		this._values   = new Set();
-		this._input    = null;
+		this._label    = null;
 		this._tags     = null;
 		this._more     = null;
 		this._search   = null;
 		this._select   = null;
-		this._listEl   = null;
+		this._listBox  = null;
 		this._active   = false;
 		this._disabled = false;
 		this._focused  = { index: -1, item: null };
 	}
 
 	connectedCallback() {
-		this._makeInputElement();
 		this._makeSelectButton();
 		const iw = document.createElement("div");
 		iw.classList.add("multiselect-wrapper");
-		this.appendChild(iw).append(this._input, this._select);
+		this.appendChild(iw).append(this._makeInputElement(), this._select);
+		this._makeListbox();
+
+		this._select.setAriaControls(this._listBox);
+		this._search.setAriaControls(this._listBox);
+		this._select.setAriaLabelledBy(this._listBox);
+
 		this.addEventListener("focusin", event => {
-			const rt = event.relatedTarget;
 			if (event.target === this._search) {
 				this.activate();
 			}
@@ -1237,6 +1389,8 @@ class Multiselect extends HTMLElement {
 		if (!this._active && !this._disabled) {
 			this.classList.add("active");
 			this._active = true;
+			this._select.ariaExpanded = true;
+			this._search.ariaExpanded = true;
 			this._search.classList.add("active");
 			this._displayList(true);
 		}
@@ -1246,8 +1400,9 @@ class Multiselect extends HTMLElement {
 		if (this._active) {
 			this.classList.remove("active");
 			this._active = false;
+			this._select.ariaExpanded = false;
+			this._search.ariaExpanded = false;
 			if (this._values.size) this._search.classList.remove("active");
-			this._search.blur();
 			this._search.value = "";
 			this._displayList(false);
 		}
@@ -1277,6 +1432,11 @@ class Multiselect extends HTMLElement {
 		}
 	}
 
+	setLabel(str) {
+		this._label = str;
+		if (this._listBox) this._listBox.ariaLabel = str;
+	}
+
 	getValues() {
 		const res = [];
 		for (const item of this._values) {
@@ -1302,18 +1462,23 @@ class Multiselect extends HTMLElement {
 	}
 
 	_makeInputElement() {
-		this._input = document.createElement("div");
-		this._input.classList.add("multiselect-input");
+		const inputEl = document.createElement("div");
+		inputEl.classList.add("multiselect-input");
 		this._tags = document.createElement("div");
+		this._tags.ariaHidden = true;
 		this._tags.classList.add("multiselect-tags");
 		this._makeSearchBar()
-		this._input.append(this._tags, this._search);
+		inputEl.append(this._tags, this._search);
+		return inputEl;
 	}
 
 	_makeSearchBar() {
 		this._search = document.createElement("input");
 		this._search.type = "text";
+		this._search.role = "textbox";
 		this._search.tabIndex = 0;
+		this._search.ariaHasPopup = "listbox";
+		this._search.ariaExpanded = false;
 		this._search.disabled = this._disabled;
 		this._search.classList.add("multiselect-search");
 		this._search.setAttribute("spellcheck", "false");
@@ -1324,24 +1489,32 @@ class Multiselect extends HTMLElement {
 		this._search.addEventListener("keydown", event => {
 			let idx = this._focused.index;
 			switch (event.code) {
+				case "Down":
 				case "ArrowDown":
 					event.preventDefault();
 					if (++idx < this._aitems.length) this._focusItem(idx, true);
 					break;
+				case "Up":
 				case "ArrowUp":
 					event.preventDefault();
 					if (--idx >= 0) this._focusItem(idx, true);
 					break;
+				case "Home":
+					event.preventDefault();
+					if (idx > 0) this._focusItem(0, true);
+					break;
+				case "End":
+					event.preventDefault();
+					if (++idx > 0 && idx < this._aitems.length) this._focusItem(this._aitems.length - 1, true);
+					break;
 				case "Enter":
 				case "NumpadEnter":
 					if (this._active) {
 						event.preventDefault();
-						if (this._focused.item) {
-							this._updateResult(this._focused.item);
-							this.deactivate();
-						}
+						if (this._focused.item) this._updateResult(this._focused.item);
 					}
 					break;
+				case "Esc":
 				case "Escape":
 					if (this._active) {
 						event.preventDefault();
@@ -1355,7 +1528,11 @@ class Multiselect extends HTMLElement {
 
 	_makeSelectButton() {
 		this._select = document.createElement("div");
+		this._select.role = "button";
 		this._select.tabIndex = -1;
+		this._select.ariaExpanded = false;
+		this._select.ariaHasPopup = "listbox";
+		if (this._disabled) this._select.ariaDisabled = true;
 		this._select.classList.add("multiselect-select");
 		this._select.addEventListener("click", event => {
 			if (this._active) {
@@ -1365,33 +1542,36 @@ class Multiselect extends HTMLElement {
 		});
 	}
 
+	_makeListbox() {
+		this._listBox = document.createElement("ul");
+		this._listBox.role = "listbox";
+		this._listBox.tabIndex = -1;
+		if (this._label) this._listBox.ariaLabel = this._label;
+		this._listBox.ariaMultiSelectable = true;
+		this._listBox.classList.add("multiselect-options", "hidden");
+		this.append(this._listBox);
+		this._listBox.addEventListener("click", event => {
+			if (event.target.role === "option") {
+				event.preventDefault();
+				this._updateResult(this._aitems.find(item => event.target === item.element));
+				this.deactivate();
+			}
+		});
+	}
+
 	_displayList(visible) {
 		if (!visible) {
-			this._listEl && this._listEl.classList.add("hidden");
+			this._listBox && this._listBox.classList.add("hidden");
 			return;
 		}
 
-		if (!this._listEl) {
-			this._listEl = document.createElement("ul");
-			this._listEl.tabIndex = -1;
-			this._listEl.classList.add("multiselect-options");
-			this.append(this._listEl);
-			this._listEl.addEventListener("click", event => {
-				if (event.target.tagName === "LI" && !event.target.classList.contains("nodata")) {
-					this._updateResult(this._aitems.find(item => event.target === item.element));
-					this.deactivate();
-					event.preventDefault();
-				}
-			});
-		} else {
-			this._listEl.classList.remove("hidden");
-			this._listEl.scrollTop = 0;
-		}
+		this._listBox.classList.remove("hidden");
+		this._listBox.scrollTop = 0;
 		this._updateList();
 	}
 
 	_updateList() {
-		while (this._listEl.firstChild) this._listEl.lastChild.remove();
+		this._listBox.replaceChildren();
 		this._aitems = [];
 
 		let cnt = 0;
@@ -1399,7 +1579,7 @@ class Multiselect extends HTMLElement {
 		this._items.forEach(item => {
 			if (!item.element) item.element = this._makeOption(item.text, this._values.has(item), false);
 			if (item.text.includes(txt)) {
-				this._listEl.appendChild(item.element);
+				this._listBox.appendChild(item.element);
 				this._aitems.push(item);
 				++cnt;
 			}
@@ -1410,26 +1590,24 @@ class Multiselect extends HTMLElement {
 		} else {
 			this._focused.index = -1;
 			this._focused.item = null;
-			this._listEl.append(this._makeOption("No items found", false, true));
+			this._listBox.append(this._makeOption("No items found", null, true));
 		}
 	}
 
 	_makeTag(item) {
 		const tb = document.createElement("i");
-		if (!this._disabled) tb.tabIndex = 0;
 		tb.addEventListener("click", event => {
 			event.preventDefault();
-			if (!this._disabled) this._updateResult(item)
-		});
-		tb.addEventListener("keydown", event => {
-			if (event.code === "Space") {
-				event.preventDefault();
-				if (!this._disabled) this._updateResult(item);
+			if (!this._disabled) {
+				this._updateResult(item)
+				this._search.focus();
 			}
 		});
 		const el = document.createElement("div");
 		el.classList.add("multiselect-tag");
-		el.appendChild(document.createElement("span")).textContent = item.text;
+		const sp = el.appendChild(document.createElement("span"));
+		sp.tabIndex = -1;
+		sp.textContent = item.text
 		el.append(tb);
 		item.tag = el;
 		return el;
@@ -1437,11 +1615,12 @@ class Multiselect extends HTMLElement {
 
 	_makeOption(text, selected, nodata) {
 		const el = document.createElement("li");
+		el.role = "option";
 		el.textContent = text;
-		if (selected) el.classList.add("selected");
 		if (nodata) {
 			el.classList.add("nodata");
 		} else {
+			el.ariaSelected = selected;
 			el.addEventListener("pointerenter", event => this._focusItem(event.target));
 		}
 		return el;
@@ -1463,14 +1642,15 @@ class Multiselect extends HTMLElement {
 		this._focused.item && this._focused.item.element.classList.remove("focused");
 		item.element.classList.add("focused");
 		this._focused = { index: idx, item: item };
-		if (scroll) scroll_to_element(item.element, this._listEl);
+		this._search.setAttribute("aria-activedescendant", item.element.getId());
+		if (scroll) scroll_to_element(item.element, this._listBox);
 	}
 
 	_clearResults() {
 		this._values.clear();
 		this._items.forEach(item => {
 			item.tag && item.tag.remove();
-			item.element && item.element.classList.remove("selected");
+			if (item.element) item.element.ariaSelected = false;
 		});
 		this._more && this._more.remove();
 		this.dispatchEvent(new Event("change"));
@@ -1481,13 +1661,13 @@ class Multiselect extends HTMLElement {
 		if (!Array.isArray(items)) items = [ items ];
 		items.forEach(item => {
 			if (this._values.delete(item)) {
-				item.element && item.element.classList.remove("selected");
+				if (item.element) item.element.ariaSelected = false;
 			} else {
 				this._values.add(item);
-				item.element && item.element.classList.add("selected");
+				if (item.element) item.element.ariaSelected = true;
 			}
 		});
-		while (this._tags.firstChild) this._tags.lastChild.remove();
+		this._tags.replaceChildren();
 		let cnt = 0;
 		for (const vi of this._values) {
 			this._tags.append(vi.tag || this._makeTag(vi));
@@ -1507,13 +1687,305 @@ class Multiselect extends HTMLElement {
 
 	_disableChanged() {
 		if (this._disabled) this.deactivate();
-		for (const item of this._values) {
-			if (item.tag) {
-				const el = item.tag.querySelector("i");
-				if (this._disabled) el.removeAttribute("tabindex"); else el.tabIndex = 0;
-			}
-		}
 		if (this._search) this._search.disabled = this._disabled;
+		if (this._select) this._select.ariaDisabled = this._disabled;
 	}
 }
 customElements.define("multi-select", Multiselect);
+
+class HintButton {
+	constructor(params) {
+		this._params = params || {};
+		this._element = null;
+		this._content = null;
+	}
+
+	element() {
+		if (!this._element) {
+			const el = document.createElement("div");
+			el.classList.add("hint-block");
+			const bt = el.appendChild((new ToolbarButton({ content: "info_icon", title: "Details" })).element());
+			bt.tabIndex = 0;
+			const ct = el.appendChild(document.createElement("div"));
+			ct.tabIndex = -1;
+			ct.classList.add("hint-content", "hidden");
+			bt.setAriaControls(ct);
+			bt.addEventListener("click", event => {
+				if (!this._content) {
+					switch (typeof(this._params.content)) {
+						case "function":
+							this._content = this._params.content(this._params.data);
+							break;
+						case "object":
+						case "string":
+							this._content = this._params.content;
+							break;
+					}
+					if (this._content) ct.append(this._content);
+				}
+				if (this._content) {
+					ct.classList.remove("hidden");
+					ct.focus();
+				}
+			});
+			ct.addEventListener("focusout", event => {
+				if (!event.relatedTarget || !ct.contains(event.relatedTarget)) {
+					ct.classList.add("hidden");
+				}
+			});
+			ct.addEventListener("keydown", event => {
+				switch (event.code) {
+					case "Esc":
+					case "Escape":
+						event.preventDefault();
+						bt.focus();
+						break;
+				}
+			});
+			this._element = el;
+		}
+		return this._element;
+	}
+
+	reset() {
+		if (this._content) {
+			this._content.remove();
+			this._content = null;
+		}
+	}
+}
+
+class MenuBar {
+	constructor(element) {
+		this._element = element;
+		this._tbtn    = null;
+		this._mbar    = null;
+		this._focused = false;
+	}
+
+	static instance() {
+		if (!this._instance) this._instance = new MenuBar(document.getElementById("main-menu-block"));
+		return this._instance;
+	}
+
+	init() {
+		this._tbtn = this._element.querySelector(".toggle-button");
+		this._tggl = this._element.querySelector('#main-menu-toggle');
+		this._mbar = this._element.querySelector('ul[role="menubar"]');
+		this._updateMenu();
+
+		function delayedFocus() {
+			this._mbar.addEventListener("transitionend", event => this.focus(), { once: true });
+		}
+
+		this._element.addEventListener("focusin", event => {
+			this._focused = true;
+		});
+		this._element.addEventListener("focusout", event => {
+			this._focused = false;
+			setTimeout(() => {
+				if (!this._focused) this._tggl.checked = false;
+			}, 0);
+		});
+		this._tbtn.addEventListener("keydown", event => {
+			switch (event.key) {
+				case " ":
+				case "Enter":
+					this._tggl.checked = !this._tggl.checked;
+					if (this._tggl.checked) delayedFocus.call(this);
+					break;
+			}
+		});
+		this._tbtn.addEventListener("click", delayedFocus.bind(this));
+		this._mbar.addEventListener("click", event => {
+			const target = event.target;
+			switch (target.role) {
+				case "menuitem":
+					if (target.ariaHasPopup === "true") {
+						this._toggleMenu(target);
+						this._focusItem(target);
+					} else {
+						this._tggl.checked = false;
+					}
+					break;
+			}
+		});
+		this._mbar.addEventListener("keydown", this._onKeydown.bind(this));
+		return this;
+	}
+
+	element(selector) {
+		if (typeof(selector) !== "string") return this._element;
+		return this._element.querySelector(selector);
+	}
+
+	focus() {
+		const that = this;
+		function findOldFocus(ul) {
+			for (const li of ul.children) {
+				if (window.getComputedStyle(li).display !== "none" && !li.classList.contains("disabled")) {
+					let item = that._getMenuItem(li);
+					if (item.tabIndex === 0) return item;
+					if (item.ariaHasPopup) {
+						item = findOldFocus(li.children[1]);
+						if (item) return item;
+					}
+				}
+			}
+		}
+		let m = findOldFocus(this._mbar);
+		if (!m) m = this._getFirstMenuItem();
+		this._focusItem(m);
+	}
+
+	updateCurrent() {
+		this._mbar.querySelectorAll('[role="menuitem"][aria-current]').forEach(el => {
+			el.ariaCurrent = null;
+		});
+		if (!this._focused) {
+			this._mbar.querySelectorAll('[role="menuitem"][aria-expanded="true"]').forEach(el => {
+				el.ariaExpanded = false;
+			});
+		}
+
+		const url = new URL(document.location);
+		url.search = "";
+		const href = url.toString();
+		for (const el of this._mbar.querySelectorAll('a[role="menuitem"]')) {
+			if (el.href === href) {
+				el.ariaCurrent = "page";
+				const pi = this._getMenuItem(el.parentNode.parentNode.parentNode);
+				if (pi) pi.ariaExpanded = true;
+			}
+		}
+		this._mbar.tabIndex = -1;
+	}
+
+	insertItem(title, href, position) {
+		const li = document.createElement("li");
+		li.role = "none";
+		const ae = li.appendChild(document.createElement("a"));
+		ae.role = "menuitem";
+		ae.tabIndex = -1;
+		ae.href = href;
+		ae.textContent = title;
+		if (position < 0 || position >= this._mbar.children.length) {
+			this._mbar.append(li);
+		} else {
+			this._mbar.insertBefore(li, this._mbar.children[position]);
+		}
+		return li;
+	}
+
+	_focusItem(item) {
+		if (item && item.role !== "menuitem") item = this._getMenuItem(item);
+		if (!item) return;
+
+		this._mbar.querySelectorAll('[role="menuitem"][tabindex="0"]').forEach(el => {
+			if (el !== item) el.tabIndex = -1;
+		});
+		item.tabIndex = 0;
+		item.focus();
+	}
+
+	_focusNextItem(cItem) {
+		this._focusItem(this._getNextMenuItem(cItem.parentNode));
+	}
+
+	_focusPreviousItem(cItem) {
+		this._focusItem(this._getPreviousMenuItem(cItem.parentNode));
+	}
+
+	_getMenuItem(li) {
+		return li.querySelector('[role="menuitem"]');
+	}
+
+	_isItemAvailable(li) {
+		return (window.getComputedStyle(li).display !== "none" && !li.classList.contains("disabled"));
+	}
+
+	_getFirstMenuItem() {
+		for (const li of this._mbar.children) {
+			if (this._isItemAvailable(li)) return li;
+		}
+	}
+
+	_getLastMenuItem() {
+		let li = this._mbar.lastElementChild;
+		while (li) {
+			if (!this._isItemAvailable(li)) return this._getPreviousMenuItem(li);
+			if (this._getMenuItem(li).ariaExpanded !== "true") return li;
+			li = li.children[1].lastElementChild;
+		}
+	}
+
+	_getNextMenuItem(li) {
+		const ci = this._getMenuItem(li);
+		if (ci.ariaExpanded === "true") {
+			const first = ci.nextElementSibling.firstElementChild;
+			if (this._isItemAvailable(first)) return first;
+			return this._getNextMenuItem(first);
+		}
+
+		while (true) {
+			let next = li.nextElementSibling;
+			while (next) {
+				if (this._isItemAvailable(next)) return next;
+				next = next.nextElementSibling;
+			}
+
+			if (li.parentNode === this._mbar) return this._getFirstMenuItem();
+			li = li.parentNode.parentNode;
+		}
+	}
+
+	_getPreviousMenuItem(li) {
+		let prev = li.previousElementSibling;
+		while (prev) {
+			if (!this._isItemAvailable(prev)) return this._getPreviousMenuItem(prev);
+			if (this._getMenuItem(prev).ariaExpanded !== "true") return prev;
+			prev = prev.children[1].lastElementChild;
+		}
+		if (li.parentNode === this._mbar) return this._getLastMenuItem();
+		return li.parentNode.parentNode;
+	}
+
+	_updateMenu() {
+		this._mbar.querySelectorAll('[role="menuitem"], [role="menu"]').forEach(el => {
+			el.tabIndex = -1;
+		});
+	}
+
+	_toggleMenu(item) {
+		item.ariaExpanded = (item.ariaExpanded !== "true");
+	}
+
+	_onKeydown(event) {
+		const tg = event.target;
+		switch (event.key) {
+			case " ":
+			case "Enter":
+				if (tg.ariaHasPopup === "true") this._toggleMenu(tg);
+				break;
+			case "Esc":
+			case "Escape":
+				this._tggl.checked = false;
+				this._tbtn.focus();
+				break;
+			case "Up":
+			case "ArrowUp":
+				this._focusPreviousItem(tg);
+				break;
+			case "Down":
+			case "ArrowDown":
+				this._focusNextItem(tg);
+				break;
+			case "Home":
+				this._focusItem(this._getFirstMenuItem());
+				break;
+			case "End":
+				this._focusItem(this._getLastMenuItem());
+				break;
+		}
+	}
+}
